import sys
#SQLAlchemy
from sqlalchemy import *
from sqlalchemy.orm import mapper, sessionmaker
from sqlalchemy.pool import NullPool
#models
from artist import Artist
from setlist import Setlist
from concert import Concert
from album import Album
from track import Track
from video import Video

#definition of the mysql database
db = create_engine('mysql://root@localhost/music_database?charset=utf8', poolclass=NullPool)

def createDB():
	"""
		If the database is not created, this method creates it.
		After that links the Classes with the tables of the database.
	"""
	metadata = MetaData(db)

	# creating a table Artist
	artist_table = Table('artist', metadata,
		Column('id', Integer, primary_key=True),
		Column('name', String(255)),
		Column('mbid', String(255)),
		Column('url', String(255)),
		Column('image', String(255)),
		Column('listeners', Integer),
		Column('playcount', Integer),
		Column('tags', Text),
		Column('bio', Text),
		Column('images',Text)
	)

	# creating a table Album
	album_table = Table('album', metadata,
		Column('id', Integer, primary_key=True),
		Column('artist_id', Integer),
		Column('name', String(255)),
		Column('mbid', String(255)),
		Column('release_date', String(255)),
		Column('url', String(255)),
		Column('cover', String(255)),
		Column('big_cover', String(255)),
		Column('listeners', Integer),
		Column('playcount', Integer),
		Column('tags', Text)
	)

	ForeignKeyConstraint(columns=[album_table.c.artist_id], refcolumns=[artist_table.c.id])

	# creating a table Track
	track_table = Table('track', metadata,
		Column('id', Integer, primary_key=True),
		Column('album_id', Integer),
		Column('name', String(255)),
		Column('mbid', String(255)),
		Column('url', String(255)),
		Column('duration', Time),
		Column('string_duration', String(255)),
		Column('listeners', Integer),
		Column('playcount', Integer),
		Column('tags', Text)
	)

	ForeignKeyConstraint(columns=[track_table.c.album_id], refcolumns=[album_table.c.id])

	# creating a table Video
	video_table = Table('video', metadata,
		Column('id', Integer, primary_key=True),
		Column('track_id', String(255)),
		Column('yt_id', String(255)),
		Column('title', String(255)),
		Column('published', Date),
		Column('url', String(255)),
		Column('duration', Time),
		Column('view_count', Integer),
		Column('rating', Float),
		Column('thumbnail', String(255))
	)

	ForeignKeyConstraint(columns=[video_table.c.track_id], refcolumns=[track_table.c.id])	

	# creating a table Concert
	concert_table = Table('concert', metadata,
		Column('id', Integer, primary_key=True),
		Column('artist_id', Integer),
		Column('name', String(255)),
		Column('popularity', Float),
		Column('venue_name', String(255)),
		Column('venue_url', String(255)),
		Column('url', String(255)),
		Column('start', TIMESTAMP),
		Column('location', String(255))
	)

	ForeignKeyConstraint(columns=[concert_table.c.artist_id], refcolumns=[artist_table.c.id])	

	#creating a table Setlist
	setlist_table = Table('setlist', metadata,
		Column('id', Integer, primary_key=True),
		Column('artist_id', Integer),
		Column('event_date', String(255)),
		Column('venue_name', String(255)),
		Column('venue_city', String(255)),
		Column('country', String(255)),
		Column('songs', Text),
		Column('event_url', String(255))
	)
	ForeignKeyConstraint(columns=[setlist_table.c.artist_id], refcolumns=[artist_table.c.id])	

	# Try to create all the defined tables.
	try:
		artist_table.create()
	except:
		print 'TABLE \'artist\' already exists.'
	try:
		album_table.create()
	except:
		print 'TABLE \'album\' already exists.'
	try:
		track_table.create()
	except:
		print 'TABLE \'track\' already exists.'
	try:
		video_table.create()
	except:
		print 'TABLE \'video\' already exists.'
	try:
		concert_table.create()
	except:
		print 'TABLE \'video\' already exists.'
	try:
		setlist_table.create()
	except:
		print 'TABLE \'video\' already exists.'

	# create a configured "Session" class.
	Session = sessionmaker(bind=db)

	# create a Session and commits the tables.
	session = Session()
	session.commit()

	metadata = MetaData(db)


	# loading definitions automatically
	artist_table = Table('artist', metadata, autoload=True)

	# loading definitions automatically
	album_table = Table('album', metadata, autoload=True)

	# loading definitions automatically
	track_table = Table('track', metadata, autoload=True)

	# loading definitions automatically
	video_table = Table('video', metadata, autoload=True)

	# loading definitions automatically
	concert_table = Table('concert', metadata, autoload=True)

	# loading definitions automatically
	setlist_table = Table('setlist', metadata, autoload=True)

	#If the database is not correct, the aplication can't work properly.
	try:
		# map the holding class to the table definition
		mapper(Artist, artist_table)

		mapper(Album, album_table)

		mapper(Track, track_table)

		mapper(Video, video_table)

		mapper(Concert, concert_table)

		mapper(Setlist, setlist_table)
	except:
		print "impossible to map the models with the database"
		sys.exit()

	session.close()
	db.dispose()