import urllib2
import json
import time
import re
#SQLAlchemy
from sqlalchemy import *
from sqlalchemy.orm import mapper, sessionmaker
from sqlalchemy.pool import NullPool
#youtube
import gdata.youtube
import gdata.youtube.service
#musicbrainz
import musicbrainz2.webservice as ws
from musicbrainz2.model import Release
#Flask
from flask import Flask, render_template, request, jsonify
#google images
from google import *
#models
from artist import Artist
from setlist import Setlist
from concert import Concert
from album import Album
from track import Track
from video import Video
#ORM
from database_init import *

#definition of the mysql database
db = create_engine('mysql://root@localhost/music_database?charset=utf8', poolclass=NullPool)
#Last.fm API key
api_key = "e7dc908a4c5b80013ca0be4b64e3f696"

def getInfo(artist):
	"""
		This function has to collect all the information of the given artist.
	"""
	# albums, concerts anf setlist info is obtained by the searchData function.
	albums = searchData(artist.id, "albums")
	concerts = searchData(artist.id, "concerts")
	setlists = searchData(artist.id, "setlists")
	info_albums = []
	info_artist = artist
	new_images = []
	"""
		Try to parse the stored data from the images of the artist.
		As data that depends on a foreign API, try statement is used to avoid external conflicts.

	"""
	try:
		info_artist.images = info_artist.images.replace("u'","'").replace("[","").replace("]","").split(",")
		for image in info_artist.images:
			image = str(image.replace(";",",").replace("'{","{").replace("}'","}"))
			new_images.append(json.loads(image))
	except:
		pass
	info_artist.images = new_images
	new_tags = []
	"""
		As with the images, add the tags to an array.
	"""
	try:
		info_artist.tags = info_artist.tags.replace("} {","};{").replace("u'","'").replace("'","\"")
		info_artist.tags = info_artist.tags.split(";")
		for tag in info_artist.tags:
			new_tags.append(json.loads(tag))
		info_artist.tags = new_tags
	except:
		pass
	auxTracks = []

	#Loops over the albums and get all its info, like tracks and videos.
	for album in albums:
		auxTracks= searchData(album.id, "tracks")
		auxTracksVid = []
		for track in auxTracks:
			videos = searchData(track.id, "videos")
			auxVideos = []
			for video in videos:
				auxVideos.append(video)
			auxTracksVid.append({"track":track, "videos": auxVideos})
		info_albums.append({"info":album, "tracks": auxTracksVid })

	auxSetLists = []
	for setlist in setlists:
		auxSetSongs = str(setlist.songs)
		auxSetSongs = setlist.songs.replace("[","").replace("]","").replace('u"','"').replace("u'","'").replace("', '",'", "').replace("}, {","};{").split(";")
		songsList = []
		for setSong in auxSetSongs:
			try:
				songsList.append(json.loads(setSong.replace("{'",'{"').replace("}, '", '"}, "').replace("': {",'": {').replace("': '", '": "').replace("': \"",'": "').replace("'}",'"}')))
			except:
				pass
		auxSetLists.append({"info":setlist,"songs":songsList});
	info = { "artist":info_artist, "albums":info_albums, "concerts":concerts, "setlists": auxSetLists}
	return info

def commitObject(obj):
	"""
		This function connects to the database and store any type of object
		that the system sends as a parameter.
	"""
	db.connect()
	metadata = MetaData(db)
	# create a configured "Session" class.
	Session = sessionmaker(bind=db)
	# create a Session
	session = Session()
	# Add the object to the database.
	session.add(obj)
	session.merge(obj)
	session.commit()
	object_id = 0
	# if is the correct type of object, get the id generated from the database.
	if obj.__class__ != Video and obj.__class__ != Concert and obj.__class__ != Setlist:
		object_id = session.query(obj.__class__).filter(obj.__class__.mbid == obj.mbid)[0].id
	session.close()
	db.dispose()
	return object_id

def artistContainer(mbid):
	"""
		This function is responsible of create the "Artist" object with the
		required parameters and call to store it to the database.
	"""
	f=urllib2.urlopen("http://ws.audioscrobbler.com/2.0/?method=artist.getinfo&mbid="+mbid+"&api_key="+api_key+"&format=json")
	artistInfo = json.loads(f.read())
	f.close()
	objectArtist = Artist(artistInfo)
	#SQLAlchemy
	artistId = commitObject(objectArtist)
	objectArtist.id = artistId;
	return objectArtist


def setlistsContainer(artistId, mbid):
	"""
		This function collects and creates an object for every setlist of the desired
		artist.
	"""
	try:
		f=urllib2.urlopen("http://api.setlist.fm/rest/0.1/search/setlists.json?artistMbid="+mbid)
		full_setlists = json.loads(f.read())
		f.close()
		if int(full_setlists["setlists"]["@total"]) > 0:
			for setlist in full_setlists["setlists"]["setlist"]:
				try:
					objectSetlist = Setlist(artistId, setlist)
					commitObject(objectSetlist)
				except:
					pass
	except:
		pass


def concertsContainer(artistId, mbid):
	"""
		This function collects and creates an object for every next concert of the desired
		artist.
	"""
	f=urllib2.urlopen("http://api.songkick.com/api/3.0/artists/mbid:"+mbid+"/calendar.json?apikey=zUDhhcXRtx8SFhPQ")
	full_concerts = json.loads(f.read())
	f.close()
	if full_concerts["resultsPage"]["results"] != {}:
		for concert in full_concerts["resultsPage"]["results"]["event"]:
			objectConcert = Concert(artistId, concert)
			commitObject(objectConcert)

def albumsContainer(artistId, mbid):
	"""
		This function gets from musicbrainz the mbid of all the albums of the requested artist
		and get all the information of every album from the last.fm api, using the mbid of the album.
	"""
	q = ws.Query()
	#only request the albums, not compilations or singles.
	mbFilter = ws.ReleaseFilter(releaseTypes = (Release.TYPE_ALBUM, Release.TYPE_OFFICIAL), artistId = str(mbid))
	mbReleases = q.getReleases(mbFilter)
	listAlbums = []
	retAlbums = []
	for a in mbReleases:
		if a.release.title not in listAlbums:
			listAlbums.append(a.release.title)
			albumMbid = a.release.id[a.release.id.find("release/")+8:]
			f=urllib2.urlopen("http://ws.audioscrobbler.com/2.0/?method=album.getinfo&mbid="+albumMbid+"&api_key="+api_key+"&format=json")
			fullAlbum = json.loads(f.read())
			f.close()
			if "error" not in fullAlbum.keys():
				objectAlbum = Album( artistId, fullAlbum )
				idAlbum = commitObject(objectAlbum)
				retAlbums.append({"id":idAlbum, "tracks":objectAlbum.tracks});
			else:
				listAlbums.pop()
	return retAlbums


def tracksContainer(idAlbum, tracks):
	"""
		This function gets from musicbrainz the mbid of all the tracks of the requested album
		and get all the information of every track from the last.fm api, using the mbid of the track.
	"""
	retTracks = []
	try:
		for t in tracks["track"]:
			trackName = t["name"]
			artistName = t["artist"]["name"]
			trackName = trackName.replace(" ", "+")
			artistName = artistName.replace(" ", "+")
			f = urllib2.urlopen("http://ws.audioscrobbler.com/2.0/?method=track.getinfo&artist="+artistName+"&track="+trackName+"&api_key="+api_key+"&format=json")
			trackInfo = json.loads(f.read())
			f.close()
			if "error" not in trackInfo:
				objectTrack = Track(idAlbum, trackInfo)
				#SQLAlchemy
				idTrack = commitObject(objectTrack)
				retTracks.append({"id": idTrack, "name": objectTrack.name})
	except:
		pass

	return retTracks


def videosContainer(idTrack, trackName, artistName):
		"""
			This function search videos on youtube for every track requested associated
			to an artist. Once the Video objects are created, stores they in the database.
		"""
		yt_service = gdata.youtube.service.YouTubeService()
		query = gdata.youtube.service.YouTubeVideoQuery()
		query.vq = trackName+" "+artistName
		query.orderby = 'relevance'
		query.racy = 'include'
		feed = yt_service.YouTubeQuery(query)

		try:
			if feed.entry.__len__() > 3:
				counter = 0
				while counter < 3:
					entry = feed.entry[counter]
					counter = counter + 1
					objectVideo = Video(idTrack, entry)
					#SQLAlchemy
					commitObject(objectVideo)
			else:
				for entry in feed.entry:
					objectVideo = Video(idTrack, entry)
					#SQLAlchemy
					commitObject(objectVideo)
		except:
			pass


def searchData(dataId, dataType):
	"""
		This function gets from the database any type of data requested.
		If the information is not in the database, returns an empty array.
	"""
	db.connect()
	metadata = MetaData(db)
	# create a configured "Session" class
	Session = sessionmaker(bind=db)
	# create a Session
	session = Session()

	try:
		if (dataType == "artist"):
			ret = session.query(Artist).filter(Artist.mbid == dataId)[0]
		elif (dataType == "albums" ):
			ret = session.query(Album).filter(Album.artist_id == dataId)
		elif (dataType == "tracks"):
			ret = session.query(Track).filter(Track.album_id == dataId)
		elif (dataType == "videos"):
			ret = session.query(Video).filter(Video.track_id == dataId)
		elif (dataType == "concerts"):
			ret = session.query(Concert).filter(Concert.artist_id == dataId)
		elif (dataType == "setlists"):
			ret = session.query(Setlist).filter(Setlist.artist_id == dataId)

	except:
		ret = []

	session.close()
	db.dispose()
	return ret

def getTrueMbid(search):
	"""
		checks the mbid got from the client with the one used in musicbrainz
		due to avoid problems finding the albums.
	"""
	search = str(search)
	q = ws.Query()
	search_mbid = q.getArtistById(search).id
	true_mbid = str(search_mbid[search_mbid.find("artist/")+7:])
	return true_mbid

def startSearch(search):
	"""
		This is the main function that receives a string with the MBID of the requested artist.
		Before starting the search tries to create the database (if is not created) and does the
		ORMapping of the different classes.
	"""
	search = getTrueMbid(search)

	inDatabase = searchData(search, "artist")
	#If the artist is already stored in the database use it to get the rest of the information. 
	searchedArtist = inDatabase
	#If not, ask for the artist info to last.fm and create a new object artist.
	if (not inDatabase):
		objectArtist = artistContainer(search)
		setlistsContainer(objectArtist.id, search)
		concertsContainer(objectArtist.id, search)
		albumsList = albumsContainer(objectArtist.id, search)
		#loops over the album and tracks to create every object.
		for album in albumsList:
			tracksList = tracksContainer(album["id"], album["tracks"])
			for track in tracksList:
				videosContainer(track["id"],track["name"], objectArtist.name)
		searchedArtist = objectArtist

	results = getInfo(searchedArtist)



	return results

app = Flask(__name__)

@app.route("/")
def index():
	"""
		Once the user connects to the main page, use the index.html template.
	"""
	return render_template('index.html')


@app.route("/search", methods=['GET', 'POST'])
def ask():
	"""
		Gets the mbid with a get request and returns the search.html with the artist info.
	"""
	if request.method == 'GET':
		mbid = request.args.get('artist', '')
		content = startSearch(mbid)
		return render_template("search.html",info = content)

if __name__ == "__main__":
	#Before start the service check if the database is correct.
	createDB()
	app.run(host='0.0.0.0', threaded = True)