/**
 * Contains all the logic that is used only in the main page.
 * @type {Object}
 */
namespace.main = {};
/**
 * Anonymous functions that encapsulates the main namespace.
 * @param  {Object} win: Alias for the DOM window object.
 * @param  {Object} ns: Alias for the namespace object.
 * @param  {Object} utils: Alias for the namespace.utilities object.
 * @param  {Object} $: Alias for the Jquery library.
 * @param  {undefined} undefined: alias for the undefined variable.
 */
(function(win, ns, utils, $, undefined){

	ns.main = {
		/**
		 * When an artist is selected modify the html and make the request to the server.
		 * @param  {Object} event: the event that launch the method.
		 */
		selectArtist: function(event){
			if (this.id == ""){
				alert("We are not able to find this artist.")
			}
			else{
				$("#resultsList").html("<h2>Loading artist...</h2>");
				win.location = "search?artist="+this.id;
				$(".floated").click(this.disableArtist);
			}
		},
		/**
		 * When a request is in progress the other artists are disabled.
		 * @param  {Object} event: the event that launch the method.
		 */
		disableArtist: function(event){
			alert("we are collecting information from another artist");
		},
		/**
		 * When the user writes a key, updates the results section of the page with the
		 * artists that match with the words of the input text.
		 * @param  {Object} event: the event that launch the method, when a key is released.
		 */
		searchArtist: function(event){
			var keyword = $("#inputArtist").val();
			keyword = keyword.replace(" ", "+");
			var apiKey = "e7dc908a4c5b80013ca0be4b64e3f696";
			var oRequest;
			oRequest = $.ajax({type: "POST",
					  url: "http://ws.audioscrobbler.com/2.0/?method=artist.search&artist=" + keyword + "&api_	key=" + apiKey + "&format=json",
					  dataType:"jsonp"});

			oRequest.success(function(){
				$("#resultsList").html("<p>Loading...</p>");
			});
			oRequest.done(function(msg){
				try{
					var artistNames = [];
					var sBlock = "";
					var sUrlImage, sMbid, sName;
					if (msg != undefined){
						for( var i = 0; i<msg["results"]["artistmatches"]["artist"].length; i++){
							if ( msg["results"]["artistmatches"]["artist"][i]["mbid"] != "" ){
								sUrlImage = msg["results"]["artistmatches"]["artist"][i]["image"][2]["#text"];
								sMbid = msg["results"]["artistmatches"]["artist"][i]["mbid"];
								sName = msg["results"]["artistmatches"]["artist"][i]["name"];
								if (sUrlImage == ""){
									sUrlImage = "http://static.tumblr.com/k9utpfa/tcom8wpif/default_cover_m.jpg";
								}
								sBlock += "<li><figure title = '" + sName + "' class = 'floated' id = '"+ sMbid +"'><img src = '"+sUrlImage+"'><figcaption>"+sName+"</figcaption></li>";
							}
						}
					}
				}
				catch(err){
					$("#resultsList").text("impossible to find artist");
				}
				//Once the request is finished, adds the html code and bind its elements with the different methods.
				$("#resultsList").html(sBlock);
				$(".floated").mousedown(utils.opacBut);
				$(".floated").mouseup(utils.noOpacBut);
				$(".floated").mouseover(utils.focusIn);
				$(".floated").mouseout(utils.focusOut);
				$(".floated").click(ns.main.selectArtist);

			});
		},
		/**
		 * Bind the method searchArtist with the event keyup on the input text box.
		 */
		init: function(){
			$("#inputArtist").keyup(this.searchArtist);
		}
	};
	/**
	 * Once the HTML is loaded, launches the init method.
	 */
	win.onload = function (){
		ns.main.init();
	};

}(window, namespace, namespace.utilities, jQuery));