/**
 * Contains most of the logic that is used only in the results page.
 * @type {Object}
 */
namespace.results = {};
/**
 * Anonymous functions that encapsulates the main namespace.
 * @param  {Object} win: Alias for the DOM window object.
 * @param  {Object} ns: Alias for the namespace object.
 * @param  {Object} utils: Alias for the namespace.utilities object.
 * @param  {Object} $: Alias for the Jquery library.
 * @param  {undefined} undefined: alias for the undefined variable.
 */
(function(win, doc, ns, utils, $, undefined){
	ns.results = {
		/**
		 * If there is no information of a section, removes it and disable it link
		 */
		removeEmptySections: function(){
			var aSections, oSection, nSectionIndex, i = 0, aMenuBar;
			aSections = $("section.section_content");
			aMenuBar = $("#menu_bar li");
			nSectionIndex = aSections.length;
			for (; i<nSectionIndex; i++){
				oSection = aSections[i];
				if (oSection.children[1].children.length == 0 || (oSection.children[1].children.length == 1 && oSection.children[1].children[0].tagName == "HR")){
					aMenuBar[i].firstChild.removeAttribute("href");
					aMenuBar[i].firstChild.className = "disabled";
					oSection.parentNode.removeChild(oSection);
				}
			}
		},
		/**
		 * Search the related tweets asyncronously and show them in the HTML.
		 */
		twitterSearch: function(){
			var sKeyWord = $("h1").text();
			var oArgument = {
				type: 'GET',
				dataType: 'jsonp',
				data:{q: sKeyWord},
				success: function(data){
						var oResultsList = doc.getElementById('content_twitter');
						var sFinalContent = "", i = 0, oDate, sDate;

						oResultsList.innerHTML = ""; //deletes all previous results in the list
						for (; i<data.results.length; i++){
							oDate = new Date(data.results[i].created_at);
							sDate = oDate.getDay()+"/"+oDate.getMonth()+"/"+oDate.getFullYear()+ " at " + oDate.getHours() + ":" + oDate.getMinutes() + ":" + oDate.getSeconds();
							sFinalContent += '<li class="tweet"><img class="tweet_image" src="'+data.results[i].profile_image_url+'"><div class = "tweet_info"><p>';
							sFinalContent += '<span class="tweet_date"> ' + sDate;
							sFinalContent += '</span><span class="tweet_user"> ' + data.results[i].from_user_name +":";
							sFinalContent += '</span><span class="tweet_text"> ' + data.results[i].text;
							sFinalContent += '</span></p></div></li>';
						}
						oResultsList.innerHTML = "<section><h2>Twitter</h2>"+sFinalContent+"</section>";
				},
			};
			oArgument.url = 'http://search.twitter.com/search.json'
			//Sends the ajax request using jQuery
			$.ajax(oArgument);
		},
		/**
		 * Bind the elements with the different methods and call the Google News and Twitter searches.
		 */
		init: function(){
			$(".section_content h2 a").click(utils.hideElement);
			$(".album_cover").click(utils.hideAlbum);
			$(".song").click(utils.hideSong);
			$(".concert_name").click(utils.hideConcert);
			$(".setlist_name").click(utils.hideSetlist);
			$(".section_content h2 a").mousedown(utils.opacBut);
			$(".album_cover").mousedown(utils.opacBut);
			$(".section_content h2 a").mouseup(utils.noOpacBut);
			$(".album_cover").mouseup(utils.noOpacBut);
			$(".album_cover").mouseover(utils.focusIn);
			$(".album_cover").mouseout(utils.focusOut);
			$(".artist_image").mouseover(utils.focusImage);
			$(".artist_image").mouseout(utils.focusImageOut);
			$(".album_cover").mouseover(utils.focusIn);
			$("a.youtube").YouTubePopup();
			ns.results.removeEmptySections();
			ns.results.twitterSearch();
		}
	};
	/**
	 * call the init method once the html is loaded.
	 */
    win.onload = function (){
        ns.results.init();
    };

}(window, window.document, namespace, namespace.utilities, jQuery)); 