/**
 * namespace contains most of the functions that are used in the application.
 * @type {Object}
 */
var namespace = namespace || {};

/**
 * Anonymous functions that encapsulates the utilities namespace.
 * @param  {Object} win: Alias for the DOM window object.
 * @param  {Object} ns: Alias for the namespace object.
 * @param  {Object} $: Alias for the Jquery library.
 * @param  {undefined} undefined: alias for the undefined variable.
 */
(function(win, ns, $, undefined){
	ns.utilities = {
		/**
		 * Hides the content of the section when clicks in its title.
		 * @param  {Object} event: the event that launch the method.
		 */
		hideElement: function(event){
			var cont = this.parentNode.nextSibling.nextSibling;
			event.preventDefault();
			if ($(cont).hasClass("hidden")){
				$(cont).removeClass("hidden");

			}
			else{
				$(cont).addClass("hidden");

			}
		},
		/**
		 * Hides the information of the selected album.
		 * @param  {Object} event: the event that launch the method.
		 */
		hideAlbum: function(event){
			var cont = this.nextSibling.nextSibling;
			event.preventDefault();
			if ($(cont).hasClass("show_album_info")){
				$(cont).removeClass("show_album_info");
			}
			else{
				$(".show_album_info").removeClass("show_album_info");
				$(cont).addClass("show_album_info");
			}
		},

		/**
		 * Hides the information of the selected track.
		 * @param  {Object} event: the event that launch the method.
		 */
		hideSong: function(event){
			var cont = this.nextSibling.nextSibling;
			event.preventDefault();
			if ($(cont).hasClass("hidden")){
				$(".track_info").addClass("hidden");
				$(cont).removeClass("hidden");
			}
			else{
				$(cont).addClass("hidden");
			}
		},
		/**
		 * Hides the information of the selected concert.
		 * @param  {Object} event: the event that launch the method.
		 */
		hideConcert: function(event){
			var cont = this.nextSibling.nextSibling;
			event.preventDefault();
			if ($(cont).hasClass("hidden")){
				$(".concert_info").addClass("hidden");
				$(cont).removeClass("hidden");
			}
			else{
				$(cont).addClass("hidden");
			}
		},
		/**
		 * Hides the information of the selected setlist.
		 * @param  {Object} event: the event that launch the method.
		 */
		hideSetlist: function(event){
			var cont = this.nextSibling.nextSibling;
			event.preventDefault();
			if ($(cont).hasClass("hidden")){
				$(".setlist_info").addClass("hidden");
				$(cont).removeClass("hidden");
			}
			else{
				$(cont).addClass("hidden");
			}
		},
		/**
		 * When the mouse is out of the element returns to its normal appearance.
		 * @param  {Object} event: the event that launch the method.
		 */
		noOpacBut: function(event){
			$(this).removeClass("opac");
		},
		/**
		 * When the mouse is over the element, its appearance change.
		 * @param  {Object} event: the event that launch the method.
		 */
		opacBut: function(event){
			$(this).addClass("opac");
		},
		/**
		 * When the mouse is over an album, its appearance change.
		 * @param  {Object} event: the event that launch the method.
		 * @return {[type]}
		 */
		focusIn: function(event){
			$(this).addClass("album_opac");
		},
		/**
		 * When the mouse is out of the album returns to its normal appearance.
		 * @param {Object} event: the event that launch the method.
		 */
		focusOut: function(event){
			$(this).removeClass("album_opac");
		},
		/**
		 * When the mouse is over an image, its appearance change.
		 * @param  {Object} event: the event that launch the method.
		 */
		focusImage: function(event){
			$(this).addClass("image_over");
		},
		/**
		 * When the mouse is out of an image returns to its normal appearance.
		 * @param  {Object} event: the event that launch the method.
		 */
		focusImageOut: function(event){
			$(this).removeClass("image_over");
		}
	};

}(window, namespace, jQuery));